using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;

namespace TFT_Common
{
    public static class JsonHelper
    {
        private static readonly JsonSerializerSettings jsonSerializerSetting = new()
        {
            ContractResolver = new Newtonsoft.Json.Serialization.DefaultContractResolver()
            {
                NamingStrategy = new Newtonsoft.Json.Serialization.SnakeCaseNamingStrategy()
            },
            NullValueHandling = NullValueHandling.Ignore
        };
        private static readonly JsonSerializerSettings ByteArrayToJsonSetting = new()
        {
            Converters = new List<JsonConverter>() {
                new JsonByteArrayConverter()
            },
            NullValueHandling = NullValueHandling.Ignore
        };
        public static string ToJsonExt(object data)
        {
            return JsonConvert.SerializeObject(data, jsonSerializerSetting);
        }
        /// <summary>
        /// лΪjsonַ
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public static string ToJson(object data)
        {
            return JsonConvert.SerializeObject(data);
        }
        public static string ToJsonExt_ByteArray(object data)
        {
            return JsonConvert.SerializeObject(data, ByteArrayToJsonSetting);
        }
        /// <summary>
        /// jsonַлΪ
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="data">json</param>
        /// <returns></returns>
        public static T ToObject<T>(string data)
        {
            return JsonConvert.DeserializeObject<T>(data);
        }
    }

    public class JsonByteArrayConverter : JsonConverter
    {
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            string result = string.Empty;
            if (value is byte[] temp)
            {
                result = BitConverter.ToString(temp);
            }
            writer.WriteValue(result);
        }
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            string strVal = reader.Value.ToString();
            if (strVal.Contains("-"))
            {
                string[] arr = strVal.Split('-');
                return arr.Select(x => byte.Parse(x)).ToArray();
            }
            else
            {
                throw new NotImplementedException();
            }
        }
        public override bool CanConvert(Type objectType)
        {
            return objectType == typeof(byte[]);
        }
    }
}
