﻿using MQTTnet;
using MQTTnet.Client;
using MQTTnet.Server;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace TFT_Common
{
    public static class MqttClientHelper
    {
        public static string Id { get; set; }
        public static string Ip { get; set; }
        public static string UserName { get; set; }
        public static string Password { get; set; }
        public static int Port { get; set; }

        public static Queue<MqttApplicationMessage> MessageQueue { get; set; } = new();

        public static IMqttClient Client { get; set; }

        public static void Init(string ip,int port)
        {
            Id = $"Id{DateTime.Now:YYMMDDHHmmss}";
            Ip = ip ;
            Port = port ;
            MessageQueue.Clear();
            Client = new MqttFactory().CreateMqttClient();
            Client.ApplicationMessageReceivedAsync += MqttClient_ApplicationMessageReceivedAsync;
            Client.ConnectedAsync += MqttClient_ConnectedAsync;
            Client.DisconnectedAsync += MqttClient_DisconnectedAsync;
        }
        public static async Task ConnectMqttServerAsync()
        {
            // Create a new MQTT client.

            if (Client == null)
            {
                var factory = new MqttFactory();
                Client = factory.CreateMqttClient();

                Client.ApplicationMessageReceivedAsync += MqttClient_ApplicationMessageReceivedAsync;
                Client.ConnectedAsync += MqttClient_ConnectedAsync;
                Client.DisconnectedAsync += MqttClient_DisconnectedAsync;
            }

            //非托管客户端
            try
            {
                //Create TCP based options using the builder.
                var options = new MqttClientOptionsBuilder()
                    .WithClientId(Id)
                    .WithTcpServer(Ip, Port)
                    //.WithCredentials(txtUsername.Text, txtPsw.Text)
                    //.WithTls()//服务器端没有启用加密协议，这里用tls的会提示协议异常
                    .WithCleanSession()
                .Build();
                await Client.ConnectAsync(options);
            }
            catch (Exception ex)
            {
            }

            //托管客户端
            try
            {
                //// Setup and start a managed MQTT client.
                //var options = new ManagedMqttClientOptionsBuilder()
                //    .WithAutoReconnectDelay(TimeSpan.FromSeconds(5))
                //    .WithClientOptions(new MqttClientOptionsBuilder()
                //        .WithClientId("Client_managed")
                //        .WithTcpServer("192.168.88.3", 8223)
                //        .WithTls()
                //        .Build())
                //    .Build();

                //var mqttClient = new MqttFactory().CreateManagedMqttClient();
                //await mqttClient.SubscribeAsync(new TopicFilterBuilder().WithTopic("my/topic").Build());
                //await mqttClient.StartAsync(options);
            }
            catch (Exception)
            {

            }
        }

        public static async Task Publish(string topic, string Message)
        {
            if (string.IsNullOrEmpty(topic))
            {
                MessageBox.Show("发布主题不能为空！");
                return;
            }

            var message = new MqttApplicationMessageBuilder()
                .WithTopic(topic)
                .WithPayload(Message)
                .WithQualityOfServiceLevel(MQTTnet.Protocol.MqttQualityOfServiceLevel.AtMostOnce)
                .WithRetainFlag(true)
                .Build();

            await Client.PublishAsync(message);
        }

        public static async Task Subscribe(string topic)
        {
            if (string.IsNullOrEmpty(topic))
            {
                MessageBox.Show("订阅主题不能为空！");
                return;
            }

            if (!Client.IsConnected)
            {
                MessageBox.Show("MQTT客户端尚未连接！");
                return;
            }

            // Subscribe to a topic
            await Client.SubscribeAsync(new MqttTopicFilterBuilder()
                .WithTopic(topic)
                .WithAtMostOnceQoS()
                .Build()
                );
        }



        private static async Task MqttClient_DisconnectedAsync(MqttClientDisconnectedEventArgs arg)
        {
            //提示连接断开

            //重新连接
            if (!Client.IsConnected)
            {
                var options = new MqttClientOptionsBuilder()
                    .WithClientId(Id)
                    .WithTcpServer(Ip, Port)
                    .WithCredentials(UserName, Password)
                    //.WithTls()
                    .WithCleanSession()
                    .Build();
                await Task.Delay(TimeSpan.FromSeconds(5));
                try
                {
                    await Client.ConnectAsync(options);
                }
                catch (Exception err)
                {
                }
            }
            else
            {

            }

        }

        private static Task MqttClient_ConnectedAsync(MqttClientConnectedEventArgs arg)
        {
            return Task.CompletedTask;
        }

        private static Task MqttClient_ApplicationMessageReceivedAsync(MqttApplicationMessageReceivedEventArgs arg)
        {
            MessageQueue.Enqueue(arg.ApplicationMessage);
            return Task.CompletedTask;
        }


    }
}
