﻿using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Principal;
using System.Text;
using System.Threading.Tasks;

namespace TFT_Model.Entities
{
    /// <summary>
    /// PLC点位配置表
    ///</summary>
    [SugarTable("bas_plc_item")]
    public class BasPlcItem
    {
        /// <summary>
        ///  
        ///</summary>
        [SugarColumn(ColumnName = "id", IsPrimaryKey = true, IsIdentity = true)]
        public long Id { get; set; }
        /// <summary>
        /// PLC地址编码 
        ///</summary>
        [SugarColumn(ColumnName = "plc_address_code")]
        public string PlcAddressCode { get; set; }
        /// <summary>
        /// 站台号 
        ///</summary>
        [SugarColumn(ColumnName = "loc_no")]
        public string LocNo { get; set; }
        /// <summary>
        /// PLCItem编码 
        ///</summary>
        [SugarColumn(ColumnName = "plc_item_code")]
        public string PlcItemCode { get; set; }
        /// <summary>
        /// PLCItem描述 
        ///</summary>
        [SugarColumn(ColumnName = "plc_item_describe")]
        public string PlcItemDescribe { get; set; }
        /// <summary>
        /// PLCItem类型。0：读；1：写；2：读和写； 
        ///</summary>
        [SugarColumn(ColumnName = "plc_item_type")]
        public int PlcItemType { get; set; }
        /// <summary>
        /// PLCItem数据类型。0：int；1：ascii；2：bin； 
        ///</summary>
        [SugarColumn(ColumnName = "plc_item_format")]
        public int PlcItemFormat { get; set; }
        /// <summary>
        /// Word字偏移量或Bit位偏移量 
        ///</summary>
        [SugarColumn(ColumnName = "word_sub_offset")]
        public int? WordSubOffset { get; set; }
        /// <summary>
        /// Word子长或Bit位长（默认1） 
        /// 默认值: 1
        ///</summary>
        [SugarColumn(ColumnName = "word_size")]
        public int WordSize { get; set; }
        /// <summary>
        /// Item是否为全字（数据类型为word时有效）。0：非全字；1：全字； 
        ///</summary>
        [SugarColumn(ColumnName = "is_full_word")]
        public int IsFullWord { get; set; }
        /// <summary>
        /// 二进制偏移量（Item是非全字时并且数据类型为word时生效）。 
        ///</summary>
        [SugarColumn(ColumnName = "bit_sub_offset")]
        public int? BitSubOffset { get; set; }
        /// <summary>
        /// 二进制长度 
        ///</summary>
        [SugarColumn(ColumnName = "bit_size")]
        public int? BitSize { get; set; }
        /// <summary>
        /// 创建者 
        ///</summary>
        [SugarColumn(ColumnName = "create_by")]
        public string CreateBy { get; set; }
        /// <summary>
        /// 创建时间 
        ///</summary>
        [SugarColumn(ColumnName = "create_time")]
        public DateTime CreateTime { get; set; }
        /// <summary>
        /// 更新者 
        ///</summary>
        [SugarColumn(ColumnName = "update_by")]
        public string UpdateBy { get; set; }
        /// <summary>
        /// 更新时间 
        ///</summary>
        [SugarColumn(ColumnName = "update_time")]
        public DateTime UpdateTime { get; set; }
        /// <summary>
        /// 备注 
        ///</summary>
        [SugarColumn(ColumnName = "remark")]
        public string Remark { get; set; }
    }

    public static class PlcItemCode
    {
        /// <summary>
        /// 准备好搬出
        /// </summary>
        public const string ReadyMoveOut = "ReadyMoveOut";
        /// <summary>
        /// 请求搬出
        /// </summary>
        public const string RequestMoveOut = "RequestMoveOut";
        /// <summary>
        /// 搬出中
        /// </summary>
        public const string MovingOut = "MovingOut";
        /// <summary>
        /// 搬出完成
        /// </summary>
        public const string MoveOutFinished = "MoveOutFinished";
        /// <summary>
        /// 满卡
        /// </summary>
        public const string MoveOutCstFulled = "MoveOutCstFulled";
        /// <summary>
        /// 准备好搬入
        /// </summary>
        public const string ReadyMoveIn = "ReadyMoveIn";
        /// <summary>
        /// 请求搬入
        /// </summary>
        public const string RequestMoveIn = "RequestMoveIn";
        /// <summary>
        /// 搬入中
        /// </summary>
        public const string MovingIn = "MovingIn";
        /// <summary>
        /// 搬入完成
        /// </summary>
        public const string MoveInFinished = "MoveInFinished";
        /// <summary>
        /// 满卡
        /// </summary>
        public const string MoveInCstFulled = "MoveInCstFulled";
    }
}
