﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TFT_Model.Other
{
    /// <summary>
    /// 自定义特性  专门用做程序内部的自动依赖注入（例如：Service和Interface、WPF窗体View和ViewModel）
    /// </summary>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = false)]
    public class AutoInjectAttribute : Attribute
    {
        public AutoInjectAttribute(Type interfaceType, InjectTypeEnum injectType, bool isWpfViewAndViewModel)
        {
            Type = interfaceType;
            InjectType = injectType;
            IsWpfViewAndViewModel = isWpfViewAndViewModel;
        }
        /// <summary>
        /// 接口或者窗体类型
        /// </summary>
        public Type Type { get; set; }

        /// <summary>
        /// 注入类型
        /// </summary>
        public InjectTypeEnum InjectType { get; set; }
        /// <summary>
        /// 是否为 WPF窗体和ViewModel相关的注入
        /// </summary>
        public bool IsWpfViewAndViewModel { get; set; }
    }
    /// <summary>
    /// 依赖注入类型
    /// </summary>
    public enum InjectTypeEnum
    {
        /// <summary>
        /// 限定作用域
        /// </summary>
        Scope,
        /// <summary>
        /// 单例
        /// </summary>
        Singleton,
        /// <summary>
        /// 瞬时
        /// </summary>
        Transient
    }
}
