﻿using Prism.Ioc;
using Prism.Mvvm;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using TFT_Model.Other;

namespace TFT_PC_FrameWork.AutoInject
{
    /// <summary>
    /// 自动依赖注入帮助类
    /// </summary>
    public static class AutoInjectHelper
    {
        public static void AddAutoDiWpfViewAndService(this IContainerRegistry containerRegistry)
        {
            var path = AppDomain.CurrentDomain.BaseDirectory;
            var assemblies = Directory.GetFiles(path, "*.dll").Select(Assembly.LoadFrom).ToList();
            foreach (var assembly in assemblies)
            {
                var types = assembly.GetTypes().Where(a => a.GetCustomAttribute<AutoInjectAttribute>() != null)
                    .ToList();
                if (types.Count <= 0) continue;
                foreach (var type in types)
                {
                    var attr = type.GetCustomAttribute<AutoInjectAttribute>();
                    if (attr?.Type == null) continue;
                    if (attr.IsWpfViewAndViewModel)
                    {
                        ViewModelLocationProvider.Register(attr.Type.ToString(), type);
                        containerRegistry.RegisterForNavigation(attr.Type, attr.Type.Name);
                    }
                    else
                    {
                        switch (attr.InjectType)
                        {
                            case InjectTypeEnum.Scope:
                                _ = containerRegistry.RegisterScoped(attr.Type, type);
                                break;
                            case InjectTypeEnum.Singleton:
                                _ = containerRegistry.RegisterSingleton(attr.Type, type);
                                break;
                            case InjectTypeEnum.Transient:
                                _ = containerRegistry.Register(attr.Type, type);
                                break;
                            default:
                                throw new ArgumentOutOfRangeException("未知异常！请联系管理员！");
                        }
                    }
                }
            }
        }
    }
}
