﻿using AutoMapper;
using Microsoft.Extensions.Logging;
using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.IO;
using TFT_Model.Entities;
using TFT_Model.Other;
using TFT_PC_FrameWork.AutoMapper;
using TFT_PC_FrameWork.Views;

namespace TFT_PC_FrameWork.ViewModels
{
    [AutoInject(typeof(MainWindow), InjectTypeEnum.Singleton, true)]
    public class MainWindowViewModel : BindableBase, IConfigureService
    {
        private readonly ILogger _logger;
        private readonly IRegionManager _regionManager;
        private readonly IMapper _mapper;

        private string _title = "Prism Application";
        private IRegionNavigationJournal journal;
        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }


        public MainWindowViewModel(IRegionManager regionManager,ILogger logger, IMapper mapper)
        {
            this._regionManager = regionManager;
            this._logger = logger;
            this._mapper = mapper;
        
            ButtonNavigateCommand = new DelegateCommand<string>(ButtonNavigate);
            BackCommand = new DelegateCommand(GoBack);
            NextCommand = new DelegateCommand(GoNext);
            LoadedCommand = new DelegateCommand(Configure);
            //Configure();
        }

        private void GoNext()
        {
            if(journal.CanGoForward)
                journal.GoForward();    
        }

        private void GoBack()
        {
            if(journal.CanGoBack)
                journal.GoBack();
        }

        private void ButtonNavigate(string strPar)
        {
            if (strPar == null || string.IsNullOrWhiteSpace(strPar))
            {
                return;
            }
            //导航页面
            _regionManager.Regions["ContentRegion"].RequestNavigate(strPar,arg=>
            {
                journal=arg.Context.NavigationService.Journal;
            });
        }

        /// <summary>
        /// 配置首页初始化参数，这个Configure是实现的接口IConfigureService中定义的方法；
        /// </summary>
        public void Configure()
        {
            //默认进入简报页面
            _regionManager.Regions["MenuRegion"].RequestNavigate("TreeMenuView");
            _regionManager.Regions["ContentRegion"].RequestNavigate("SysRealTimeRun");
        }
        #region 命令绑定
        public DelegateCommand<string> ButtonNavigateCommand { set; get; }
        public DelegateCommand NextCommand { set; get; }
        public DelegateCommand BackCommand { set; get; }
        public DelegateCommand LoadedCommand { set; get; }
        #endregion
    }
    public interface IConfigureService
    {
        void Configure();
    }
}
