﻿using TFT_BizService;
using Prism.Regions;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TFT_Model.Entities;
using TFT_Model.Other;
using TFT_PC_FrameWork.Assets;
using TFT_PC_FrameWork.Models;
using TFT_PC_FrameWork.Views;

namespace TFT_PC_FrameWork.ViewModels
{
    [AutoInject(typeof(TreeMenuView), InjectTypeEnum.Singleton, true)]
    public class TreeMenuViewModel
    {
        public List<MenuItemModel> Menus { get; set; } = new List<MenuItemModel>();
        // 列表，没有树型结构
        private List<MenuMd> origMenus = null;
        private readonly IRegionManager _regionManager;
        public TreeMenuViewModel(IRegionManager regionManager, IMenuService menuService)
        {
            _regionManager = regionManager;
            // 需要获取菜单数据
            origMenus = menuService.QueryList(); ;
            FillMenus(Menus, 0);
        }

        public  string EscapeUnicode( string str)  // 将字符串编码为unicode转义序列(\uxxxx)
        {
            StringBuilder tmp = new StringBuilder();
            for (int i = 0; i < str.Length; i++)
            {
                ushort uxc = (ushort)str[i];
                tmp.Append(@"\u" + uxc.ToString("x4"));
            }
            return (tmp.ToString());
        }

        public static string String2Unicode(string source)
        {
            var bytes = Encoding.Unicode.GetBytes(source);
            var stringBuilder = new StringBuilder();
            for (var i = 0; i < bytes.Length; i += 2)
            {
                stringBuilder.AppendFormat("\\u{0:x2}{1:x2}", bytes[i + 1], bytes[i]);
            }
            return stringBuilder.ToString();
        }

        ///递归
        ///
        private void FillMenus(List<MenuItemModel> menus, int parentId)
        {
            var sub = origMenus.Where(m => m.ParentId == parentId).OrderBy(o => o.Index);
            if (sub.Any())
            {
                foreach (var item in sub)
                {
                    string icon = string.Empty;
                    
                    if (!string.IsNullOrEmpty(item.MenuIcon))
                    {
                        icon = AssetHelper.FindIcon(item.MenuHeader);
                    }
                    MenuItemModel mm = new MenuItemModel(_regionManager)
                    {
                        MenuHeader = item.MenuHeader,
                        MenuIcon =icon,
                        TargetView = item.TargetView
                    };
                    menus.Add(mm);
                    FillMenus(mm.Children = new List<MenuItemModel>(), item.MenuId);
                }
            }
        }
    }
}
