﻿using Prism.Regions;
using System;
using System.Drawing;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;
using TFT_PC_FrameWork.Assets;
using TFT_PC_FrameWork.ViewModels;

namespace TFT_PC_FrameWork.Views
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private DispatcherTimer ShowTimer;
        private MainWindowViewModel _viewModel;
        public MainWindow(MainWindowViewModel viewModel)
        {
            InitializeComponent(); 
            MaxHeight = SystemParameters.WorkArea.Height + 10; //不遮挡任务栏
            WindowState = WindowState.Maximized; //默认最大化                                
            btnMax.Content = AssetHelper.FindIcon("最大化全屏");

            //添加timer
            ShowTimer = new DispatcherTimer();
            ShowTimer.Tick += new EventHandler(ShowTime);
            ShowTimer.Interval = new TimeSpan(0, 0, 0, 1, 0);
            ShowTimer.Start();
            _viewModel = viewModel;
        }
        public void ShowTime(object sender, EventArgs e)
        {
            //获得年月日 
            txtTime.Text = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult boxResult = MessageBox.Show("确认退出系统?", "确认", MessageBoxButton.OKCancel, MessageBoxImage.Question);
            if (boxResult == MessageBoxResult.OK)
            {
                Close();
                Environment.Exit(0);
            }
        }

        private void btnMax_Click(object sender, RoutedEventArgs e)
        {
            if (WindowState == WindowState.Maximized)
            {
                WindowState = WindowState.Normal;
                btnMax.Content = AssetHelper.FindIcon("最大化正常");
            }
            else
            {
                MaxHeight = SystemParameters.WorkArea.Height + 10;
                WindowState = WindowState.Maximized;
                btnMax.Content = AssetHelper.FindIcon("最大化全屏");
            }
        }

        private void btnMin_Click(object sender, RoutedEventArgs e)
        {
            WindowState = WindowState.Minimized;
        }

        private void Window_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
                DragMove();
        }
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            _viewModel.Configure();
        }
    }
}
